@@ -1,24 +1,27 @@
 /*
- * Copyright (c) 1998-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.2 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
- * Please see the License for the specific language governing rights and 
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
+
 #include <UserNotification/KUNCUserNotifications.h>
 
 #include <IOKit/IOKitKeys.h>
@@ -27,7 +30,6 @@
 #include <IOKit/usb/IOUSBDevice.h>
 #include <IOKit/usb/IOUSBControllerV2.h>
 #include <IOKit/usb/IOUSBLog.h>
-#include <IOKit/usb/IOUSBRootHubDevice.h>
 #include "AppleUSBHubPort.h"
 
 #define super OSObject
@@ -71,7 +73,7 @@ AppleUSBHubPort::init( AppleUSBHub *parent, int portNum, UInt32 powerAvailable,
     _runLock = IOLockAlloc();
     if (!_runLock)
         return kIOReturnError;
-
+        
     _initThread = thread_call_allocate((thread_call_func_t)PortInitEntry, (thread_call_param_t)this);
     if (!_initThread)
     {
@@ -154,7 +156,7 @@ AppleUSBHubPort::stop(void)
 	    }
 	    else
 	    {
-		USBLog(2, "AppleUSBHubPort[%p]::stop - trying command sleep, port %d on hub %p (%d/%d).", this,_portNum, _hub,_statusChangedThreadActive, _initThreadActive);
+		USBLog(2, "AppleUSBHubPort[%p]::stop - trying command sleep (%d/%d).", this,_statusChangedThreadActive, _initThreadActive);
 		_inCommandSleep = true;
 		if (_statusChangedThreadActive)
 		    gate->commandSleep(&_statusChangedThreadActive, THREAD_UNINT);
@@ -303,6 +305,7 @@ IOReturn
 AppleUSBHubPort::AddDevice(void)
 {
     IOReturn		err = kIOReturnSuccess;
+    bool		checkingForDeadHub = false;
 
     USBLog(5, "***** AppleUSBHubPort[%p]::AddDevice - port %d on hub %p - start", this, _portNum, _hub);
     do
@@ -315,7 +318,7 @@ AppleUSBHubPort::AddDevice(void)
             if (!_devZero)
             {
            	USBLog(2, "***** AppleUSBHubPort[%p]::AddDevice - port %d on hub %p - bus %p - unable to get devZero lock", this, _portNum, _hub, _bus);
-               FatalError(/*FIXME*/0, "acquiring device zero");
+		FatalError(/*FIXME*/0, "acquiring device zero");
                 break;
             }
         }
@@ -326,10 +329,16 @@ AppleUSBHubPort::AddDevice(void)
 
         USBLog(5, "***** AppleUSBHubPort[%p]::AddDevice - port %d on hub %p - resetting port", this, _portNum, _hub);
         SetPortVector(&AppleUSBHubPort::AddDeviceResetChangeHandler, kHubPortBeingReset);
-        if ((err = _hub->SetPortFeature(kUSBHubPortResetFeature, _portNum)))
+        err = _hub->SetPortFeature(kUSBHubPortResetFeature, _portNum);
+        if (err)
         {
             USBLog(3, "***** AppleUSBHubPort[%p]::AddDevice - port %d on hub %p - unable (err = %x) to reset port", this, _portNum, _hub, err);
             FatalError(err, "set feature (resetting port)");
+	    if (err == kIOUSBTransactionTimeout)
+	    {
+		_hub->CallCheckForDeadHub();			// pull the plug if it isn't already
+		checkingForDeadHub = true;
+	    }
             break;
         }
         USBLog(5, "***** AppleUSBHubPort[%p]::AddDevice - port %d on hub %p - sleeping 100 ms", this, _portNum, _hub);
@@ -342,14 +351,22 @@ AppleUSBHubPort::AddDevice(void)
         // Need to disable the port before releasing the lock
         //
         
-        if ( (err = _hub->ClearPortFeature(kUSBHubPortEnableFeature, _portNum)) )
-        {
-            // If we get an error at this point, it probably means that the hub went away.  Note the error but continue to 
-            // release the devZero lock.
-            //
-            USBLog(5, "***** AppleUSBHubPort[%p]::AddDevice  ClearPortFeature for Port %d returned 0x%x", this, _portNum, err);
-            FatalError(err, "clearing port feature");
-        }
+	if (!checkingForDeadHub)
+	{
+	    if ( (err = _hub->ClearPortFeature(kUSBHubPortEnableFeature, _portNum)) )
+	    {
+		// If we get an error at this point, it probably means that the hub went away.  Note the error but continue to 
+		// release the devZero lock.
+		//
+		USBLog(5, "***** AppleUSBHubPort[%p]::AddDevice  ClearPortFeature for Port %d returned 0x%x", this, _portNum, err);
+		FatalError(err, "clearing port feature");
+		if (err == kIOUSBTransactionTimeout)
+		{
+		    _hub->CallCheckForDeadHub();			// pull the plug if it isn't already
+		    checkingForDeadHub = true;
+		}
+	    }
+	}
         
         _bus->ReleaseDeviceZero();
         _devZero = false;
@@ -370,26 +387,26 @@ AppleUSBHubPort::RemoveDevice(void)
     bool			ok;
     const IORegistryPlane 	* usbPlane;
     IOUSBDevice			*cachedPortDevice;
-        
-    
-    if (_portDevice) 
+
+
+    if (_portDevice)
     {
         // Cache our port device so that we can set it to NULL so we don't reenter
         // this code (due to above check)
         //
         cachedPortDevice = _portDevice;
         _portDevice = NULL;
-        
+
         USBLog(3, "AppleUSBHubPort[%p]::RemoveDevice start (%s)", this, cachedPortDevice->getName());
         usbPlane = cachedPortDevice->getPlane(kIOUSBPlane);
-        
+
         if ( usbPlane )
             cachedPortDevice->detachAll(usbPlane);
 
         cachedPortDevice->terminate(kIOServiceRequired);
         cachedPortDevice->release();
     }
-    
+
     InitPortVectors();
 }
 
@@ -400,22 +417,22 @@ AppleUSBHubPort::SuspendPort( bool suspend)
     IOReturn 		err = kIOReturnSuccess;
     IOUSBHubPortStatus	status;
       
-    USBLog(5, "AppleUSBHubPort[%p]::SuspendPort for port %d", this, _portNum);
+    USBLog(5, "AppleUSBHubPort[%p]::SuspendPort(%d) for port %d", this, suspend, _portNum);
 
     do {
-
+        
         // If resuming, need to check that the port was suspended to begin with
         //
         err = _hub->GetPortStatus(&status, _portNum);
         if (kIOReturnSuccess != err)
         {
-            IOLog("AppleUSBHubPort::SuspendPort Could not get Port Status: 0x%x\n", err);
+            USBLog(3,"AppleUSBHubPort[%p]::SuspendPort Could not get Port Status: 0x%x", this, err);
             break;
         }
-        
+
         if (!suspend && !(status.statusFlags & kHubPortSuspend) )
         {
-            // We were trying to resume but the port was not supended.  Just ignore the 
+            // We were trying to resume but the port was not supended.  Just ignore the
             // request.
             break;
         }
@@ -423,25 +440,42 @@ AppleUSBHubPort::SuspendPort( bool suspend)
         // OK, set up the handler for the set/clear suspend feature
         //
         SetPortVector(&AppleUSBHubPort::HandleSuspendPortHandler, kHubPortSuspend);
-        
+
         if (suspend)
         {
             err = _hub->SetPortFeature(kUSBHubPortSuspendFeature, _portNum);
             if ( err != kIOReturnSuccess )
             {
                 // Root Hub failed for lucent bug
-                // 
+                //
                 // Callback device with the resume message
-                USBLog(3, "AppleUSBHubPort[%p]::SuspendPort got error = 0x%x", this, err);
+                USBLog(3, "AppleUSBHubPort[%p]::SuspendPort Could not SetPortFeature (%d) (kUSBHubPortSuspendFeature): (0x%x)", this, _portNum, err);
                 if ( _portDevice )
                     _portDevice->message(kIOUSBMessagePortHasBeenResumed, NULL, 0);
-	    }
+
+            }
+            // Send a message to our device indicating that we completed the SetPortFeature
+            if ( _portDevice )
+                _portDevice->message(kIOUSBMessagePortHasBeenSuspended, NULL, &err);
         }
         else
+        {
             err = _hub->ClearPortFeature(kUSBHubPortSuspendFeature, _portNum);
-            
-            
+            if ( err != kIOReturnSuccess )
+            {
+                USBLog(3, "AppleUSBHubPort[%p]::SuspendPort Could not ClearPortFeature (%d) (kUSBHubPortSuspendFeature): (0x%x)", this, _portNum, err);
+            }
+        }
+
+
     } while (false);
+
+    if ( err != kIOReturnSuccess )
+    {
+        // Set the handler back to default
+        //
+        SetPortVector(&AppleUSBHubPort::DefaultSuspendChangeHandler, kHubPortSuspend);
+    }
     
     return err;
 }
@@ -459,6 +493,75 @@ AppleUSBHubPort::ReEnumeratePort(UInt32 options)
     return AddDevice();
 }
 
+IOReturn 
+AppleUSBHubPort::ClearTT(bool multiTTs, UInt32 options)
+{
+IOReturn 		err = kIOReturnSuccess;
+UInt8 	 deviceAddress;  //<<0
+UInt8	 endpointNum;    //<<8
+UInt8 	 endpointType;	 //<<16 // As split transaction. 00 Control, 10 Bulk
+UInt8 	 IN;		 //<<24 // Direction, 1 = IN, 0 = OUT};
+UInt32 opts= options;
+
+UInt16 wValue, wIndex;
+IOUSBDevRequest request;
+
+    deviceAddress = options & 0xff;
+    options >>= 8;
+    
+    endpointNum = options & 0xff;
+    options >>= 8;
+    
+    endpointType = options & 0xff;
+    options >>= 8;
+    
+    IN = options & 0xff;
+    options >>= 8;
+
+/* 
+3..0 Endpoint Number
+10..4 Device Address
+12..11 Endpoint Type
+14..13 Reserved, must be zero
+15 Direction, 1 = IN, 0 = OUT
+*/
+    wValue = 0;
+    wValue = endpointNum & 0xf;
+    wValue |= (deviceAddress & 0x7f) << 4;
+    wValue |= (endpointType & 0x3) << 11;
+    wValue |= (IN & 0x1) << 15;
+    
+    if(multiTTs)
+    {
+	wIndex = _portNum;
+    }
+    else
+    {
+	wIndex = 1;
+    }
+    
+    request.bmRequestType = 0x23;
+    request.bRequest = 8;
+    request.wValue = wValue;
+    request.wIndex = wIndex;
+    request.wLength = 0;
+    request.pData = NULL;
+    request.wLenDone = 0;
+
+    err = _hub->DoDeviceRequest(&request);
+    USBLog(5,"AppleUSBHubPort[%p]::ClearTT -- port %d, options:%lX, wValue:%X, wIndex:%X, IOReturn: %LX",this, _portNum, opts, wValue, wIndex, err);
+    
+    if( (err == kIOReturnSuccess) && (endpointType == 0) )	// Control endpoint.
+    {
+	wValue ^= (1 << 15);	// Flip direction bit and do it again.
+	request.wValue = wValue;
+	err = _hub->DoDeviceRequest(&request);
+	USBLog(5,"AppleUSBHubPort[%p]::ClearTT -- do it again for control transactions, wValue:%X, IOReturn: %LX",this, wValue, err);
+    }
+    
+    return err;
+}
+
 IOReturn 
 AppleUSBHubPort::ResetPort()
 {
@@ -492,7 +595,7 @@ AppleUSBHubPort::ResetPort()
             err = _hub->ClearPortFeature(kUSBHubPortSuspendFeature, _portNum);
             if (kIOReturnSuccess != err)
             {
-                IOLog("AppleUSBHubPort::ResetPort Could not ClearPortFeature (%d) (kHubPortSuspend): (0x%x)\n", _portNum, err);
+                USBLog(3, "AppleUSBHubPort[%p]::ResetPort Could not ClearPortFeature (%d) (kHubPortSuspend): (0x%x)", this, _portNum, err);
                 break;
             }
         }
@@ -503,12 +606,19 @@ AppleUSBHubPort::ResetPort()
         SetPortVector(&AppleUSBHubPort::HandleResetPortHandler, kHubPortBeingReset);
         
         err = _hub->SetPortFeature(kUSBHubPortResetFeature, _portNum);
-        if(err != kIOReturnSuccess)
+        if (err != kIOReturnSuccess)
+        {
+            USBLog(3, "AppleUSBHubPort[%p]::ResetPort Could not ClearPortFeature (%d) (kUSBHubPortResetFeature): (0x%x)", this, _portNum, err);
+
+            // Return our vector to the default handler
+            //
+            SetPortVector(&AppleUSBHubPort::DefaultResetChangeHandler, kHubPortBeingReset);
             break;
+        }
 
     } while (false);
 
-    if(err == kIOReturnSuccess)
+    if (err == kIOReturnSuccess)
     {
         _bus->WaitForReleaseDeviceZero();
     }
@@ -582,7 +692,7 @@ IOUSBControllerV2 *v2Bus;
  **
  **********************************************************************/
 IOReturn 
-AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags)
+AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
 {
     IOReturn		err = kIOReturnSuccess;
     IOReturn		err2 = kIOReturnSuccess;
@@ -596,6 +706,19 @@ AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags)
     {
         if (_state != hpsDeadDeviceZero)
         {
+            
+            // Before doing anything, check to see if the device is really there
+            //
+            if ( !(statusFlags & kHubPortConnection) )
+            {
+                // We don't have a connection on this port anymore.
+                //
+                USBLog(5, "AppleUSBHubPort[%p]::DefaultConnectionChangeHandler - port %d - device has gone away", this, _portNum);
+                _state = hpsDeadDeviceZero;
+                err = kIOReturnNoDevice;
+                break;
+            }
+
             // in the Mac OS 9 state machine (called resetChangeHandler) we skip states 1-3
             // if we are in DeadDeviceZero state (so that we end up setting the address )
             
@@ -826,20 +949,25 @@ AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags)
                                     _hub->_device->GetAddress(), _portNum);
         if ( !err )
         {
-                _portDevice = usbDevice;
+	    _portDevice = usbDevice;
         }
         else
         {
             USBLog(3, "**9** AppleUSBHubPort[%p]::AddDeviceResetChangeHandler -  port %d, did NOT get _portDevice", this, _portNum);
             err = err2;
+	    usbDevice->release();
+	    usbDevice = NULL;
         }
         
 	if (!_portDevice)
 	{
 	    // OOPS- The device went away from under us. Probably due to an unplug. Well, there is nothing more
 	    // for us to do, so just return
-            USBLog(3, "**9** AppleUSBHubPort[%p]::AddDeviceResetChangeHandler - port %d, _portDevice disappeared, returning", this, _portNum);
+            USBLog(3, "**9** AppleUSBHubPort[%p]::AddDeviceResetChangeHandler - port %d, _portDevice disappeared, cleaning up", this, _portNum);
+            _retryPortStatus = true;
 
+            SetPortVector(&AppleUSBHubPort::DefaultResetChangeHandler, kHubPortBeingReset);
+            _hub->ClearPortFeature(kUSBHubPortEnableFeature, _portNum);
             //  Not in 9 
             if (_devZero)
             {
@@ -847,7 +975,12 @@ AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags)
                 _bus->ReleaseDeviceZero();
                 _devZero = false;
             }
-	    return kIOReturnSuccess;
+
+            USBLog(3, "**9** AppleUSBHubPort[%p]::AddDeviceResetChangeHandler - port %d, delaying 10 ms and calling AddDevice", this, _portNum);
+            IOSleep(10); // Nine waits for only 1ms
+            err = AddDevice();		
+            return err;
+	    // return kIOReturnSuccess;
 	}
 	
         // In MacOS 9, we attempt to get the full DeviceDescriptor at this point, if we had missed it earlier.  On X, we do NOT do this
@@ -895,10 +1028,8 @@ AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags)
     return err;
 }
 
-OSMetaClassDefineReservedUsed(IOUSBController,  13);
-
 IOReturn 
-AppleUSBHubPort::HandleResetPortHandler(UInt16 changeFlags)
+AppleUSBHubPort::HandleResetPortHandler(UInt16 changeFlags, UInt16 statusFlags)
 {
     IOReturn		err = kIOReturnSuccess;
     UInt32		delay = 10;
@@ -1165,7 +1296,7 @@ AppleUSBHubPort::HandleResetPortHandler(UInt16 changeFlags)
 }
 
 IOReturn
-AppleUSBHubPort::HandleSuspendPortHandler(UInt16 changeFlags)
+AppleUSBHubPort::HandleSuspendPortHandler(UInt16 changeFlags, UInt16 statusFlags)
 {
     USBLog(3, "AppleUSBHubPort[%p]::HandleSuspendPortHandler for port %d, changeFlags: 0x%x", this, _portNum, changeFlags);
     SetPortVector(&AppleUSBHubPort::DefaultSuspendChangeHandler, kHubPortSuspend);
@@ -1178,46 +1309,30 @@ AppleUSBHubPort::HandleSuspendPortHandler(UInt16 changeFlags)
 }
 
 IOReturn 
-AppleUSBHubPort::DefaultOverCrntChangeHandler(UInt16 changeFlags)
+AppleUSBHubPort::DefaultOverCrntChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
 {
     IOUSBHubDescriptor		hubDescriptor;
     bool			individualPortPower = FALSE;
     UInt16			characteristics;
     IOUSBHubPortStatus		portStatus;
     IOReturn			err;
-    IOUSBRootHubDevice		 *roothubdevice;
 
     USBLog(5, "AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler. Port %d", this,  _portNum );
     err = _hub->GetPortStatus(&portStatus, _portNum);
 
     if ( err == kIOReturnSuccess )
     {
-        if ( _hub && _hub->_device )
+        if ( (portStatus.statusFlags & kHubPortOverCurrent))
         {
-            // Workaround for #3205416:  Don't display an overcurrent if we are the root hub
-            //
-            roothubdevice = OSDynamicCast(IOUSBRootHubDevice, _hub->_device);
+            USBLog(1, "AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler. OverCurrent condition in Port %d", this,  _portNum );
+            hubDescriptor = _hub->GetCachedHubDescriptor();
 
-            if ( roothubdevice == NULL )
-            {
-                if ( (portStatus.statusFlags & kHubPortOverCurrent))
-                {
-                    USBLog(1, "AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler. OverCurrent condition in Port %d", this,  _portNum );
-                    hubDescriptor = _hub->GetCachedHubDescriptor();
+            characteristics = USBToHostWord(hubDescriptor.characteristics);
 
-                    characteristics = USBToHostWord(hubDescriptor.characteristics);
+            if ( (characteristics & 0x18) == 0x8 )
+                individualPortPower = TRUE;
 
-                    if ( (characteristics & 0x18) == 0x8 )
-                        individualPortPower = TRUE;
-
-                    DisplayOverCurrentNotice( individualPortPower );
-                }
-            }
-            else
-            {
-                USBLog(3,"AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler. Ignoring overcurrent condition on Root Hub");
-                return kIOReturnUnsupported;
-            }
+            DisplayOverCurrentNotice( individualPortPower );
         }
         else
         {
@@ -1232,7 +1347,7 @@ AppleUSBHubPort::DefaultOverCrntChangeHandler(UInt16 changeFlags)
 
 
 IOReturn 
-AppleUSBHubPort::DefaultResetChangeHandler(UInt16 changeFlags)
+AppleUSBHubPort::DefaultResetChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
 {
     USBLog(5, "AppleUSBHubPort[%p]::DefaultResetChangeHandler for port %d returning kIOReturnSuccess", this, _portNum);
     return kIOReturnSuccess;
@@ -1241,7 +1356,7 @@ AppleUSBHubPort::DefaultResetChangeHandler(UInt16 changeFlags)
 
 
 IOReturn 
-AppleUSBHubPort::DefaultSuspendChangeHandler(UInt16 changeFlags)
+AppleUSBHubPort::DefaultSuspendChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
 {
     USBLog(5, "AppleUSBHubPort[%p]::DefaultSuspendChangeHandler for port %d returning kIOReturnSuccess", this, _portNum);
     return kIOReturnSuccess;
@@ -1250,7 +1365,7 @@ AppleUSBHubPort::DefaultSuspendChangeHandler(UInt16 changeFlags)
 
 
 IOReturn 
-AppleUSBHubPort::DefaultEnableChangeHandler(UInt16 changeFlags)
+AppleUSBHubPort::DefaultEnableChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
 {
     IOReturn		err = kIOReturnSuccess;
     IOUSBHubPortStatus	status;
@@ -1275,12 +1390,12 @@ AppleUSBHubPort::DefaultEnableChangeHandler(UInt16 changeFlags)
 
 
 IOReturn 
-AppleUSBHubPort::DefaultConnectionChangeHandler(UInt16 changeFlags)
+AppleUSBHubPort::DefaultConnectionChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
 {
     IOReturn	err = kIOReturnSuccess;
     IOUSBHubPortStatus	status;
 
-    USBLog(5, "AppleUSBHubPort[%p]::DefaultConnectionChangeHandler - handling port %d changes (%d).", this, _portNum, changeFlags);
+    USBLog(5, "AppleUSBHubPort[%p]::DefaultConnectionChangeHandler - handling port %d changes (%x).", this, _portNum, changeFlags);
     _connectionChangedState = 0;
     do
     {
@@ -1428,6 +1543,15 @@ AppleUSBHubPort::PortStatusChangedHandler(void)
                 goto errorExit;
             }
 
+            // If a PC Card has been ejected, we might receive 0xff as our status
+            //
+            if ( (_portStatus.statusFlags == 0xffff) && (_portStatus.changeFlags == 0xffff) )
+            {
+                err = kIOReturnNoDevice;
+                goto errorExit;
+            }
+            
+
             _statusChangedState = 2;
             USBLog(5,"AppleUSBHubPort[%p]::PortStatusChangedHandler - port %d - status(%8x)/change(%8x) - clearing retryPortStatus", this, _portNum, _portStatus.statusFlags, _portStatus.changeFlags);
                         
@@ -1475,11 +1599,17 @@ AppleUSBHubPort::PortStatusChangedHandler(void)
             goto errorExit;
         }
 
+        if ( (_portStatus.statusFlags == 0xffff) && (_portStatus.changeFlags == 0xffff) )
+        {
+            err = kIOReturnNoDevice;
+            goto errorExit;
+        }
+
         _statusChangedState = 7;
         USBLog(5, "AppleUSBHubPort[%p]::PortStatusChangedHandler - port %d - status(%8x) - change(%8x) - before call to (%d) handler function", this, _portNum, _portStatus.statusFlags, _portStatus.changeFlags, which);
             
         _statusChangedState = ((which+1) * 20) + 1;
-        err = (this->*_changeHandler[which].handler)(_portStatus.changeFlags);
+        err = (this->*_changeHandler[which].handler)(_portStatus.changeFlags, _portStatus.statusFlags);
         _statusChangedState = ((which+1) * 20) + 2;
         USBLog(5,"AppleUSBHubPort[%p]::PortStatusChangedHandler - port %d - err (%x) on return from  call to (%d) handler function", this, _portNum, err, which);
 
@@ -1502,7 +1632,8 @@ AppleUSBHubPort::PortStatusChangedHandler(void)
             USBLog(3,"AppleUSBHubPort[%p]::PortStatusChangedHandler - port %d - error %x from (%d) handler", this, _portNum, err, which);
             break;
         }
-    } while (true);
+    }
+    while (true);
 
 errorExit:
 	
@@ -1599,7 +1730,7 @@ AppleUSBHubPort::ReleaseDevZeroLock()
     {
         (void) _hub->ClearPortFeature(kUSBHubPortEnableFeature, _portNum);
         
-        USBError(1,"AppleUSBHubPort[%p]::ReleaseDevZeroLock for port (%d) force releasing lock", this, _portNum);
+        USBError(1, "AppleUSBHubPort::ReleaseDevZeroLock()");
         _state = hpsNormal;
  
         if ( _bus )
@@ -1791,34 +1922,9 @@ void
 AppleUSBHubPort::DisplayOverCurrentNotice(bool individual)
 {
     if ( individual )
-    {
-        if ( _hub && _hub->_device )
-            IOLog("USB Notification:  The device \"%s\" has caused an overcurrent condition.  The port it is attached to has been disabled\n",_hub->_device->getName());
-        
-        KUNCUserNotificationDisplayNotice(
-                                          0,		// Timeout in seconds
-                                          0,		// Flags (for later usage)
-                                          "",		// iconPath (not supported yet)
-                                          "",		// soundPath (not supported yet)
-                                          "/System/Library/Extensions/IOUSBFamily.kext",		// localizationPath
-                                          "USB OverCurrent Header",					// the header
-                                          "USB Individual OverCurrent Notice",				// the notice - look in Localizable.strings
-                                          "OK");
-    }
+        _hub->_device->DisplayUserNotification(kUSBIndividualOverCurrentNotificationType);
     else
-    {
-        if ( _hub && _hub->_device )
-            IOLog("USB Notification:  The device \"%s\" has caused an overcurrent condition.  The hub it is attached to has been disabled\n",_hub->_device->getName());
+        _hub->_device->DisplayUserNotification(kUSBGangOverCurrentNotificationType);
 
-        KUNCUserNotificationDisplayNotice(
-                                          0,		// Timeout in seconds
-                                          0,		// Flags (for later usage)
-                                          "",		// iconPath (not supported yet)
-                                          "",		// soundPath (not supported yet)
-                                          "/System/Library/Extensions/IOUSBFamily.kext",		// localizationPath
-                                          "USB OverCurrent Header",					// the header
-                                          "USB Gang OverCurrent Notice",				// the notice - look in Localizable.strings
-                                          "OK");
-    }
     return;
 }
