@@ -1,24 +1,27 @@
 /*
- * Copyright (c) 1998-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.2 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
- * Please see the License for the specific language governing rights and 
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
+
 #ifndef _IOKIT_APPLEUSBHUB_H
 #define _IOKIT_APPLEUSBHUB_H
 
@@ -61,7 +64,8 @@ class AppleUSBHub : public IOService
     IOUSBPipe * 		_interruptPipe;
     IOBufferMemoryDescriptor *	_buffer;
     IOCommandGate *		_gate;
-
+    IOWorkLoop *		_workLoop;
+    UInt32			_locationID;
     UInt32			_inStartMethod;
     bool			_portSuspended;
     bool			_hubHasBeenDisconnected;
@@ -84,6 +88,8 @@ class AppleUSBHub : public IOService
     UInt8			_readBytes;
     UInt8			_numCaptive;
     AppleUSBHubPort **	   	_ports;		// Allocated at runtime
+    bool			_multiTTs;	// Hub is multiTT capable, and configured.
+    bool			_hsHub;		// Hub is connected to a HS bus
     IOTimerEventSource *      	_timerSource;
     UInt32			_timeoutFlag;
     UInt32			_portTimeStamp[32];
@@ -141,12 +147,13 @@ class AppleUSBHub : public IOService
     void 		PrintHubDescriptor(IOUSBHubDescriptor *desc);
 
     void 		FatalError(IOReturn err, char *str);
-    void 		DisplayNotEnoughPowerNotice();
     IOReturn		DoPortAction(UInt32 type, UInt32 portNumber, UInt32 options );
     void		StartWatchdogTimer();
     void		StopWatchdogTimer();
     IOReturn		RearmInterruptRead();
     void		ResetMyPort();
+    void		CallCheckForDeadHub(void);
+
     IOUSBHubDescriptor 	GetCachedHubDescriptor() { return _hubDescriptor; }
     
 public:
@@ -166,6 +173,7 @@ public:
     virtual bool 	terminateClient( IOService * client, IOOptionBits options );
 
     virtual IOUSBDevice * GetDevice(void) { return _device; }
+
 };
 
 
